package com.m2pool.chat.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.m2pool.chat.dto.ChatRoomDto;
import com.m2pool.chat.entity.ChatRoom;
import com.m2pool.chat.mapper.ChatMessageMapper;
import com.m2pool.chat.mapper.ChatRoomMapper;
import com.m2pool.chat.service.ChatRoomService;
import com.m2pool.chat.vo.CharRoomVo;
import com.m2pool.chat.vo.RoomPageVo;
import com.m2pool.chat.vo.RoomVo;
import com.m2pool.common.core.Result.R;
import com.m2pool.common.core.constant.HttpStatus;
import com.m2pool.common.core.utils.PageUtils;
import com.m2pool.common.core.web.page.TableDataInfo;
import com.m2pool.common.security.utils.SecurityUtils;
import com.m2pool.system.api.RemoteUserService;
import com.m2pool.system.api.entity.SysUser;
import io.jsonwebtoken.lang.Collections;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.messaging.simp.user.SimpUserRegistry;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Random;

@Service
public class ChatRoomServiceImpl extends ServiceImpl<ChatRoomMapper, ChatRoom> implements ChatRoomService {

    @Autowired
    private ChatRoomMapper chatRoomMapper;

    @Resource
    private ChatMessageMapper chatMessageMapper;

    @Resource
    private RemoteUserService remoteUserService;

    @Autowired
    private SimpUserRegistry userRegistry;


    @Override
    public TableDataInfo<ChatRoomDto> findRoomList(RoomPageVo roomPageVo) {
        String userEmail = SecurityUtils.getUsername();
        PageHelper.startPage(1, 20);
        //1.查找当前客服对应的聊天室
        List<ChatRoomDto> roomList = chatRoomMapper.findRoomList(userEmail,roomPageVo.getSendDateTime());

        PageUtils.clearPage();
//        if (roomList.isEmpty()){
//            TableDataInfo tableDataInfo = new TableDataInfo();
//            tableDataInfo.setCode(HttpStatus.ERROR);
//            tableDataInfo.setMsg("当前客服暂无聊天室");
//            return tableDataInfo;
//        }
        return getDataTable(roomList);
    }

    private TableDataInfo<ChatRoomDto> getDataTable(List<ChatRoomDto> list)
    {
        TableDataInfo<ChatRoomDto> rspData = new TableDataInfo<ChatRoomDto>();
        rspData.setCode(HttpStatus.SUCCESS);
        rspData.setRows(list);
        rspData.setMsg("查询成功");
        PageInfo<ChatRoomDto> pageInfo = new PageInfo<ChatRoomDto>(list);
        rspData.setTotal(pageInfo.getTotal());
        rspData.setTotalPage(pageInfo.getPages());
        return rspData;
    }


    @Override
    @Transactional
    public R<ChatRoomDto> findRoomByUserid(RoomVo roomVo) {
        //1.查询当前用户与对应用户是否已存在创建的聊天室
        String userEmail = roomVo.getEmail();
        ChatRoomDto roomByUserEmail = chatRoomMapper.findRoomByUserEmail(userEmail);
        if(roomByUserEmail != null){
            return R.success(roomByUserEmail);
        }

        //2.不存在创建一个聊天室
        List<SysUser> data = remoteUserService.getCSList().getData();
        if(Collections.isEmpty(data)){
            return R.fail("客服人数不足");
        }
        Random random = new Random();
        SysUser sysUser = data.get(random.nextInt(data.size()));
        ChatRoom build = ChatRoom.builder().userOneEmail(userEmail).userTwoEmail(sysUser.getEmail()).build();
        int insert = chatRoomMapper.insert(build);
        if (insert > 0){
            return R.success(ChatRoomDto.builder().id(build.getId()).selfEmail(userEmail).userEmail(build.getUserTwoEmail()).build());
        }
        return R.fail("聊天室不存在,并且创建聊天室失败");
    }

    @Override
    public R<String> updateRoom(CharRoomVo charRoomVo) {
        int i = chatRoomMapper.updateById(ChatRoom.builder().id(charRoomVo.getId()).flag(charRoomVo.getFlag()).build());
        if(i > 0){
            return R.success("修改成功");
        }
        return R.fail("修改失败,不存在该聊天室");
    }
}
